#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    restretto - REST resource endpoints testing toolkit
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from . import assertions
from . import errors
from . import utils
from .rest import Resource, Session
from .loader import load
